@@ -8,7 +8,7 @@ from logging import handlers
 from functools import wraps
 from textwrap import shorten
 from datetime import datetime
-
+from typing import Optional, Callable
 DATEFORMAT_STD = "%Y-%m-%d %H:%M:%S"
 DATEFORMAT_DEV = "%H:%M:%S"
 DATEFORMAT_FILE = "%Y-%m-%d_%H-%M-%S"
@@ -125,6 +125,11 @@ class CustomFormatter(logging.Formatter):
     3. Shortens 'funcName' and 'filename' so to not cause issues with the formatting.
     """
 
+    def __init__(self, fmt: Optional[str] = None, datefmt: Optional[str] = None, style='%', validate: bool = True, error_formatter: Callable = None, no_newline_on_error: bool = False) -> None:
+        super().__init__(fmt=fmt, datefmt=datefmt, style=style, validate=validate)
+        self.error_formatter = error_formatter
+        self.no_newline_on_error = no_newline_on_error
+
     def format(self, record):
         record.name = record.name.removeprefix('main.')
 
@@ -143,13 +148,15 @@ class CustomFormatter(logging.Formatter):
 
         s = super().format(record)
         if record.exc_text:
-            s = s.replace('\n', '') + 'ERROR ->>'
+            if self.no_newline_on_error is True:
+                s = s.replace('\n', '')
         return s
 
     def formatException(self, ei) -> str:
         output = super().formatException(ei)
-        # return '<br>'.join(line.strip() for line in output.splitlines() if line.strip() != '')
-        return repr(output)
+        if self.error_formatter is None:
+            return output
+        return self.error_formatter(output)
 
 
 class ExcludeProfilingStreamHandler(logging.StreamHandler):
@@ -159,42 +166,49 @@ class ExcludeProfilingStreamHandler(logging.StreamHandler):
             return super().emit(record)
 
 
-def _each_run_file_rotate_handler(in_file_name, in_back_up):
+def marked_exception_formatter(in_text: str) -> str:
+    text = f"| EXCEPTION START |\n\n{in_text}\n\n| EXCEPTION END |"
+    return text
+
+
+def _each_run_file_rotate_handler(in_file_name, in_back_up, formatter_kwargs: dict = None):
     _date_format = DATEFORMAT_DEV if os.getenv('IS_DEV') == "1" else DATEFORMAT_STD
-    formatter = CustomFormatter(fmt='{asctime}.{msecs:03.0f} |{levelname:^11}|{threadName:^12}|{lineno:^5}| {name:<50} |{funcName:^30}||--> {message}', datefmt=_date_format, style='{')
+    formatter = CustomFormatter(fmt='{asctime}.{msecs:03.0f} |{levelname:^11}|{threadName:^12}|{lineno:^5}| {name:<50} |{funcName:^30}||--> {message}', datefmt=_date_format, style='{', **formatter_kwargs)
     handler = handlers.RotatingFileHandler(in_file_name, mode='a', backupCount=in_back_up, maxBytes=10 * (1024 * 1024))
     handler.namer = std_namer
     handler.setFormatter(formatter)
     return handler
 
 
-def _stdout_handler():
+def _stdout_handler(formatter_kwargs: dict = None):
     _date_format = DATEFORMAT_DEV
-    formatter = CustomFormatter(fmt='{asctime} |{levelname:^11}| {name:<50} |{funcName:^30}||--> {message}', datefmt=_date_format, style='{')
+    formatter = CustomFormatter(fmt='{asctime} |{levelname:^11}| {name:<50} |{funcName:^30}||--> {message}', datefmt=_date_format, style='{', **formatter_kwargs)
     # formatter = logging.Formatter(fmt='[{asctime}][{levelname:^7}][{name:<30}][{funcName:^25}] --> {message}', datefmt=_date_format, style='{')
     handler = logging.StreamHandler()
     handler.setFormatter(formatter)
     return handler
 
 
-def _minimal_stdout_handler():
+def _minimal_stdout_handler(formatter_kwargs: dict = None):
     _date_format = DATEFORMAT_DEV
-    formatter = CustomFormatter(fmt='{asctime} |{levelname:^11}||--> {message}', datefmt=_date_format, style='{')
+    formatter = CustomFormatter(fmt='{asctime} |{levelname:^11}||--> {message}', datefmt=_date_format, style='{', **formatter_kwargs)
     # formatter = logging.Formatter(fmt='[{asctime}][{levelname:^7}][{name:<30}][{funcName:^25}] --> {message}', datefmt=_date_format, style='{')
     handler = ExcludeProfilingStreamHandler()
     handler.setFormatter(formatter)
     return handler
 
 
-def main_logger(in_file_name, in_level='DEBUG', in_back_up=2, other_logger_names=None, log_to='both', stdout_formatter: str = None):
+def main_logger(in_file_name, in_level='DEBUG', in_back_up=2, other_logger_names=None, log_to='both', stdout_formatter: str = None, formatter_kwargs: dict = None):
+    default_formatter_kwargs = {"error_formatter": marked_exception_formatter, "no_newline_on_error": False}
+    formatter_kwargs = default_formatter_kwargs if formatter_kwargs is None else default_formatter_kwargs | formatter_kwargs
     stdout_fomatter_map = {'minimal': _minimal_stdout_handler,
                            'normal': _stdout_handler}
     stdout_formatter = 'minimal' if stdout_formatter is None else stdout_formatter.casefold()
     stdout_handler_function = stdout_fomatter_map.get(stdout_formatter, _minimal_stdout_handler)
     log_to = log_to.casefold()
     should_roll_over = os.path.isfile(in_file_name)
-    run_rotating_file_handler = _each_run_file_rotate_handler(in_file_name=in_file_name, in_back_up=in_back_up)
-    stdout_handler = stdout_handler_function()
+    run_rotating_file_handler = _each_run_file_rotate_handler(in_file_name=in_file_name, in_back_up=in_back_up, formatter_kwargs=formatter_kwargs)
+    stdout_handler = stdout_handler_function(formatter_kwargs=formatter_kwargs)
     _out = logging.getLogger('main')
     all_loggers = [_out] + list(map(logging.getLogger, other_logger_names)) if other_logger_names is not None else [_out]
     for logger in all_loggers:
